#  Retrives the Powershell Path location and assigns it to a variable
$scriptPath = (Get-Location).Path

# Import the dependencies from the Common folder. You will call methods from these .ps1 files.
. $scriptPath\Common\env_conf.ps1
. $scriptPath\Common\GetSqlConnection.ps1
. $scriptPath\Common\GetRows.ps1
. $scriptPath\Common\generateRowLink.ps1
. $scriptPath\Common\SendEmail.ps1

# Acquires the HTML template to be filled with the rows from the Templates folder
$template=Get-Content -Path $scriptPath\templates\Template.html

# Retrieves the object generated by the function in env_conf.ps1
$conf = getConf

# The function converts the 12 character numeric Date in a readable format
function convertDate($numericDate){

    return $numericDate.Substring(6,2)+'/'+$numericDate.Substring(4,2)+'/'+$numericDate.Substring(0,4)

}

<# Function to create the HTML table from the retrieved rows for the template .
Accepts as arguments the row's list and a dictionary containing a map
of each row and the related link ($links). The main function will create such dictionary.#>

function createTable($rows,$links){
    # Prepares the table header's HTML with the fields' name.
    $table =  "
        <style>
        {font-family: Helvetica; font-size: 10pt;}
        TABLE{border: 1px solid black; border-collapse: collapse; font-size:11pt;}
        TH{border: 1px solid black; background: #d6feff; padding: 5px; color: #1c4d8a; font-size:13pt;}
        TD{border: 1px solid black; padding: 5px;}
        </style>
        <table>
        <tr>
        <th>Invoice ID</th>
        <th>CustomerName</th>
        <th>EmissionDate</th>
        <th>Item</th>
        <th>Quantity</th>
        <th>UnitPrice</th>
        <th>VAT</th>
        <th>Total</th>
        </tr>
    "
    # $rows contains the fields to be reported and formatted in the HTML template
    $rows |foreach { # foreach repeates the operation for each retrieved row from the query.
        # The script rounds the prices and formats Date and VAT before placing them in the HTML.
        $openedDate = convertDate($_.EmissionDate.ToString())
        $convertedPercentage =  $_.VAT.ToString("P")
        $roundedTotal = [math]::Round($_.Total, 2)
        $roundedUnitPrice = [math]::Round($_.UnitPrice, 2)
        <# Formats the rows's fields in HTML by replacing the values within
        the $tablerow string. Each variable corresponds to a table's field.
        Exception made for $_.Link: contains the result of the =GENERATEROWLINK()#>
        $tableRow = " 
            <tr>
            <td style='text-align:center;'><a href="""+$links[$_.IdEntry]+""">"+$_.IdEntry+"</a></td>
            <td>"+$_.CustomerName+"</td>
            <td>"+$openedDate+"</td>
            <td style='text-align:center;'>"+$_.Item+"</td>
            <td style='text-align:center;'>"+$_.Quantity+"</td>
            <td style='text-align:center;'>"+$roundedUnitPrice+"&#8364</td>
            <td style='text-align:center;'>"+$convertedPercentage+"%</td>
            <td style='text-align:center;'>"+$roundedTotal+"&#8364</td>
            </tr>
        "
        # Each row formatted in HTML is recorded in order in the $table variable
        $table += $tableRow 
    }
    <#After having iterated the function on all the rows, 
    the function returns the variable containg the HTML table with all the rows#>
    return $table
}

# The function replaces the _TABLE_ element in the HTML with the formatted rows
function addTemplate($table){
    return $template.Replace("__TABLE__",$table)
}


<#Adds further configuration parameters that aren't common to different scripts.
That is why you write them here instead in the env_conf. 
env_conf contains only the settings common to all custom scripts.

Settings changing from app to app are the Application's ID (applicationId), the mail's subject,
or the application's SQL table schema.

Replace each value according to your database, application, and email account.#>

$applicationId="02b28572-d66c-467b-a8f8-2046755b097c"
$schema="finance001"
$tableName="TBL_INVOICES"
$appBusinessKey = "IdEntry"
$toEmail="your.email@email.com"
$mailSubject="Weekly Reminder | Invoices"
$commonSchema=$conf.CommonSchema


<# Defines a variable containing the SQL query as string.
Use the [elegere].[F_APP_BIGINT2DATETIME] to convert a Date field value
in a format that you can use in a comparison.

Change the query according to your needs. 
The script retrieves the database's schema from env_conf ($commonSchema).#>
$rowsQuery = "			
    SELECT * FROM [$schema].TBL_DOCS_REQUEST WHERE [$commonSchema].[F_APP_BIGINT2DATETIME](DT_CREATION)>=DATEADD(d,-7,GETDATE());
"

<# This is the main function that the script will run.
It recalls functions defined above and methods from Common folder.#>
function main() {
    #Get the Connection String from the object provided by env_conf.ps1
    $connection=GetSqlConnection $conf.SqlConnectionString
    #Opens the SQL connection
    $connection.Open()
    # Tries to get the rows satisfying the query from the $rowsQuery variable and format the email.
    try
    {
        # Retrives the rows according to the query in the $rowsQuery variable.        
        $rows = GetRows $connection $rowsQuery
        # Applies the =GENERATEROWLINK() function to each row and adds a further field to the formatted table with the link.
        $rowLinks=@{}
        $rows |foreach{
            $link=GenerateRowLink $connection   $conf.HostBaseUrl  $applicationId  $conf.HostSecretKey $schema $tableName "IdEntry" $_.IdEntry
            $rowLinks.Add($_.IdEntry,$link)
        }
        # Uses the createTableRow function to create the table with the rows 
        $rowsTable=createTable $rows $rowLinks
        # Gets the template and replaces _TABLE_ with the formatted table containi
        $emailBody=addTemplate $rowsTable
        <# Generates the email notification from the formatted HTML and sends it. 
        Requires the parameters containing all the information to connect to the STMP and send the email.#>
        sendEmail $conf.MailServer $conf.MailPort $conf.MailEnableSSL $conf.MailUsername $conf.MailPassword $conf.MailFrom $toEmail $mailSubject $emailBody

        # Returns a confirmation message to the user in the Log.
        $message = @{
            Message = "Success"
            ReturnCode = "200"
          }
          $result = New-Object psobject -Property $message
        }
    catch [Exception] # If an exception happens, the script reports the error message in the log.
    {
        echo $_.Exception.Message
        $message = @{
            Message = $_.Exception.Message
            ReturnCode = "500"
          }
        $result = New-Object psobject -Property $message
    }
    finally # Whatever is the result, closes the SQL connection. 
    {
        $connection.Close()
        $connection.Dispose()
       
    }
    return $result
}

# Launches the function 'main'
main